""" Migration to alter PidXpath validation.

Generated by Django 3.2.18 on 2023-03-27
"""

from django.db import migrations, models

import core_linked_records_app.utils.dict


class Migration(migrations.Migration):
    """Migration class."""

    dependencies = [
        ("core_linked_records_app", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="pidxpath",
            name="xpath",
            field=models.CharField(
                max_length=255,
                validators=[
                    core_linked_records_app.utils.dict.validate_dot_notation
                ],
            ),
        ),
    ]
