{% extends 'core_main_app/user/sharing/modals/multi-link.html' %}

{% block modal_title %}Share PIDs{% endblock %}
{% block modal_id %}pid-list-sharing-modal{% endblock %}
{% block modal_input_id %}pid-list-sharing-link{% endblock %}
{% block modal_submit_id %}pid-list-sharing-submit{% endblock %}