""" Migration to change PidXpath designation into PidPath.

Generated by Django 4.2.3 on 2023-11-14
"""

from django.db import migrations


class Migration(migrations.Migration):
    """Migration class."""

    dependencies = [
        ("core_linked_records_app", "0003_create_permissions"),
    ]

    operations = [
        migrations.RenameModel("PidXpath", "PidPath"),
        migrations.RenameField("PidPath", "xpath", "path"),
    ]
